function x = solve_cs_1d(A,b,method,noise_sigma)

% Solve 1-D L-1 norm minimization problem. with noise bound
% the constraint Ax=b changes to |Ax-b|<sigma
%
% The minimization problem is:
%
%   min f(x)
%   subject to |Ax-b|<sigma
%
% method=0: least square, f(x) = ||x||_2
%
% method=1: nonnegative least square, f(x) = ||x||_2, and x>=0
%
% method=2: L1 norm, with nonnegative constraint
%                   f(x) = ||x||_1, and x>=0
%
% method=3: L1 norm, gradient, with nonnegative constraint
%                   f(x) = ||grad(x)||_1, and x>=0
%
% method=4: L1 norm, gradient and value, with nonnegative constraint 
%
%                   f(x) = ||x||_1 + lambda*||grad(x)||_1, and x>=0
%                   (default: lambda=1)   
%
%
% Jinwei Gu
% 2010/2/26

options = optimset('Display', 'off');

if nargin<4
	noise_sigma = 0; %1e-10;
end

if method==0
% method 0: least square
    x = A\b;
    x(x<0) = 0;

elseif method==1        
% method 1: nonnegative least square
    x = lsqnonneg(A, b);

elseif method==2
% method 2: cs, with non-negative constraint 
    ny = size(A,2);
%    x = linprog(ones(ny,1), [], [], A, b, zeros(ny,1), [], [], options);

    x = linprog(ones(ny,1), [A;-A], [b+noise_sigma; -b+noise_sigma], ...
			[], [], zeros(ny,1), [], [], options);

elseif method==3
% method 3: cs, non-negative, gradient
    [k, ny] = size(A);
    A1 = zeros(k,2*ny-1);
    A1(:,1:ny) = A;
    A2 = zeros(2*(ny-1),2*ny-1);
    b2 = zeros(2*(ny-1),1);
    for i = 1:ny-1
        A2(2*i-1, i)    = 1;
        A2(2*i-1, i+1)  = -1;
        A2(2*i-1, ny+i) = -1;

        A2(2*i, i)    = -1;
        A2(2*i, i+1)  = 1;
        A2(2*i, ny+i) = -1;
    end
    f = zeros(2*ny-1,1);
    f(ny+1:end) = 1;
    %f(ny+1:end) = trimf(linspace(0,1,ny-1), [0,0.5,1]);
    x = linprog(f, [A2;A1;-A1], [b2;b+noise_sigma;-b+noise_sigma], ...
			[], [], zeros(2*ny-1,1), [], [], options);
    x = x(1:ny);

elseif method==4
% method 4: cs, non-negative, gradient+data
    [k, ny] = size(A);
    A1 = zeros(k,2*ny-1);
    A1(:,1:ny) = A;
    A2 = zeros(2*(ny-1),2*ny-1);
    b2 = zeros(2*(ny-1),1);
    for i = 1:ny-1
        A2(2*i-1, i)    = 1;
        A2(2*i-1, i+1)  = -1;
        A2(2*i-1, ny+i) = -1;

        A2(2*i, i)    = -1;
        A2(2*i, i+1)  = 1;
        A2(2*i, ny+i) = -1;
    end
    %lambda = 5.0;
    lambda = 1.0;  % Jinwei Gu. 2010/2/26
    f = ones(2*ny-1,1);
    f(ny+1:end) = lambda;
    %f(1:ny) = trimf(linspace(0,1,ny), [0,0.5,1]);
    %f(ny+1:end) = lambda*trimf(linspace(0,1,ny-1), [0,0.5,1]);
    x = linprog(f, [A2;A1;-A1], [b2;b+noise_sigma;-b+noise_sigma], ...
			[], [], zeros(2*ny-1,1), [], [], options);
    x = x(1:ny);    
else
    disp('invaid method')
end
